@@ -10,7 +10,7 @@ use crate::{
     archive,
     commands::warn_user_about_loading_zip_in_memory,
     extension::{split_first_compression_format, CompressionFormat::*, Extension},
-    utils::{user_wants_to_continue, FileVisibilityPolicy},
+    utils::{io::lock_and_flush_output_stdio, user_wants_to_continue, FileVisibilityPolicy},
     QuestionAction, QuestionPolicy, BUFFER_CAPACITY,
 };
 
@@ -56,6 +56,10 @@ pub fn compress_files(
                 encoder,
                 level.map_or_else(Default::default, |l| bzip2::Compression::new((l as u32).clamp(1, 9))),
             )),
+            Bzip3 => Box::new(
+                // Use block size of 16 MiB
+                bzip3::write::Bz3Encoder::new(encoder, 16 * 2_usize.pow(20))?,
+            ),
             Lz4 => Box::new(lz4_flex::frame::FrameEncoder::new(encoder).auto_finish()),
             Lzma => Box::new(xz2::write::XzEncoder::new(
                 encoder,
@@ -69,16 +73,21 @@ pub fn compress_files(
                     .from_writer(encoder),
             ),
             Zstd => {
-                let zstd_encoder = zstd::stream::write::Encoder::new(
+                let mut zstd_encoder = zstd::stream::write::Encoder::new(
                     encoder,
                     level.map_or(zstd::DEFAULT_COMPRESSION_LEVEL, |l| {
                         (l as i32).clamp(zstd::zstd_safe::min_c_level(), zstd::zstd_safe::max_c_level())
                     }),
-                );
-                // Safety:
-                //     Encoder::new() can only fail if `level` is invalid, but the level
-                //     is `clamp`ed and therefore guaranteed to be valid
-                Box::new(zstd_encoder.unwrap().auto_finish())
+                )?;
+                // Use all available PHYSICAL cores for compression
+                zstd_encoder.multithread(num_cpus::get_physical() as u32)?;
+                Box::new(zstd_encoder.auto_finish())
+            }
+            Brotli => {
+                let default_level = 11; // Same as brotli CLI, default to highest compression
+                let level = level.unwrap_or(default_level).clamp(0, 11) as u32;
+                let win_size = 22; // default to 2^22 = 4 MiB window size
+                Box::new(brotli::CompressorWriter::new(encoder, BUFFER_CAPACITY, level, win_size))
             }
             Tar | Zip | Rar | SevenZip => unreachable!(),
         };
@@ -92,9 +101,9 @@ pub fn compress_files(
     }
 
     match first_format {
-        Gzip | Bzip | Lz4 | Lzma | Snappy | Zstd => {
+        Gzip | Bzip | Bzip3 | Lz4 | Lzma | Snappy | Zstd | Brotli => {
             writer = chain_writer_encoder(&first_format, writer)?;
-            let mut reader = fs::File::open(&files[0]).unwrap();
+            let mut reader = fs::File::open(&files[0])?;
 
             io::copy(&mut reader, &mut writer)?;
         }
@@ -104,8 +113,11 @@ pub fn compress_files(
         }
         Zip => {
             if !formats.is_empty() {
-                warn_user_about_loading_zip_in_memory();
+                // Locking necessary to guarantee that warning and question
+                // messages stay adjacent
+                let _locks = lock_and_flush_output_stdio();
 
+                warn_user_about_loading_zip_in_memory();
                 if !user_wants_to_continue(output_path, question_policy, QuestionAction::Compression)? {
                     return Ok(false);
                 }
@@ -124,13 +136,19 @@ pub fn compress_files(
             io::copy(&mut vec_buffer, &mut writer)?;
         }
         Rar => {
-            archive::rar::no_compression_notice();
-            return Ok(false);
+            #[cfg(feature = "unrar")]
+            return Err(archive::rar::no_compression());
+
+            #[cfg(not(feature = "unrar"))]
+            return Err(archive::rar_stub::no_support());
         }
         SevenZip => {
             if !formats.is_empty() {
-                warn_user_about_loading_sevenz_in_memory();
+                // Locking necessary to guarantee that warning and question
+                // messages stay adjacent
+                let _locks = lock_and_flush_output_stdio();
 
+                warn_user_about_loading_sevenz_in_memory();
                 if !user_wants_to_continue(output_path, question_policy, QuestionAction::Compression)? {
                     return Ok(false);
                 }
