@@ -6,6 +6,8 @@ mod list;
 
 use std::{ops::ControlFlow, path::PathBuf};
 
+use bstr::ByteSlice;
+use decompress::DecompressOptions;
 use rayon::prelude::{IndexedParallelIterator, IntoParallelRefIterator, ParallelIterator};
 use utils::colors;
 
@@ -14,31 +16,32 @@ use crate::{
     cli::Subcommand,
     commands::{compress::compress_files, decompress::decompress_file, list::list_archive_contents},
     error::{Error, FinalError},
-    extension::{self, parse_format},
-    info,
+    extension::{self, parse_format_flag},
     list::ListOptions,
-    utils::{self, to_utf, EscapedPathDisplay, FileVisibilityPolicy},
-    warning, CliArgs, QuestionPolicy,
+    utils::{
+        self, colors::*, is_path_stdin, logger::info_accessible, path_to_str, EscapedPathDisplay, FileVisibilityPolicy,
+    },
+    CliArgs, QuestionPolicy,
 };
 
 /// Warn the user that (de)compressing this .zip archive might freeze their system.
 fn warn_user_about_loading_zip_in_memory() {
-    const ZIP_IN_MEMORY_LIMITATION_WARNING: &str = "\n\
-        \tThe format '.zip' is limited and cannot be (de)compressed using encoding streams.\n\
-        \tWhen using '.zip' with other formats, (de)compression must be done in-memory\n\
-        \tCareful, you might run out of RAM if the archive is too large!";
+    const ZIP_IN_MEMORY_LIMITATION_WARNING: &str = "\n  \
+        The format '.zip' is limited by design and cannot be (de)compressed with encoding streams.\n  \
+        When chaining '.zip' with other formats, all (de)compression needs to be done in-memory\n  \
+        Careful, you might run out of RAM if the archive is too large!";
 
-    warning!("{}", ZIP_IN_MEMORY_LIMITATION_WARNING);
+    eprintln!("{}[WARNING]{}: {ZIP_IN_MEMORY_LIMITATION_WARNING}", *ORANGE, *RESET);
 }
 
 /// Warn the user that (de)compressing this .7z archive might freeze their system.
 fn warn_user_about_loading_sevenz_in_memory() {
-    const SEVENZ_IN_MEMORY_LIMITATION_WARNING: &str = "\n\
-        \tThe format '.7z' is limited and cannot be (de)compressed using encoding streams.\n\
-        \tWhen using '.7z' with other formats, (de)compression must be done in-memory\n\
-        \tCareful, you might run out of RAM if the archive is too large!";
+    const SEVENZ_IN_MEMORY_LIMITATION_WARNING: &str = "\n  \
+        The format '.7z' is limited by design and cannot be (de)compressed with encoding streams.\n  \
+        When chaining '.7z' with other formats, all (de)compression needs to be done in-memory\n  \
+        Careful, you might run out of RAM if the archive is too large!";
 
-    warning!("{}", SEVENZ_IN_MEMORY_LIMITATION_WARNING);
+    eprintln!("{}[WARNING]{}: {SEVENZ_IN_MEMORY_LIMITATION_WARNING}", *ORANGE, *RESET);
 }
 
 /// This function checks what command needs to be run and performs A LOT of ahead-of-time checks
@@ -50,6 +53,13 @@ pub fn run(
     question_policy: QuestionPolicy,
     file_visibility_policy: FileVisibilityPolicy,
 ) -> crate::Result<()> {
+    if let Some(threads) = args.threads {
+        rayon::ThreadPoolBuilder::new()
+            .num_threads(threads)
+            .build_global()
+            .unwrap();
+    }
+
     match args.cmd {
         Subcommand::Compress {
             files,
@@ -66,7 +76,7 @@ pub fn run(
             // Formats from path extension, like "file.tar.gz.xz" -> vec![Tar, Gzip, Lzma]
             let (formats_from_flag, formats) = match args.format {
                 Some(formats) => {
-                    let parsed_formats = parse_format(&formats)?;
+                    let parsed_formats = parse_format_flag(&formats)?;
                     (Some(formats), parsed_formats)
                 }
                 None => (None, extension::extensions_from_path(&output_path)),
@@ -109,7 +119,7 @@ pub fn run(
                 // having a final status message is important especially in an accessibility context
                 // as screen readers may not read a commands exit code, making it hard to reason
                 // about whether the command succeeded without such a message
-                info!(accessible, "Successfully compressed '{}'.", to_utf(&output_path));
+                info_accessible(format!("Successfully compressed '{}'", path_to_str(&output_path)));
             } else {
                 // If Ok(false) or Err() occurred, delete incomplete file at `output_path`
                 //
@@ -130,14 +140,18 @@ pub fn run(
                 }
             }
 
-            compress_result?;
+            compress_result.map(|_| ())
         }
-        Subcommand::Decompress { files, output_dir } => {
+        Subcommand::Decompress {
+            files,
+            output_dir,
+            remove,
+        } => {
             let mut output_paths = vec![];
             let mut formats = vec![];
 
             if let Some(format) = args.format {
-                let format = parse_format(&format)?;
+                let format = parse_format_flag(&format)?;
                 for path in files.iter() {
                     let file_name = path.file_name().ok_or_else(|| Error::NotFound {
                         error_title: format!("{} does not have a file name", EscapedPathDisplay::new(path)),
@@ -162,6 +176,7 @@ pub fn run(
 
             // The directory that will contain the output files
             // We default to the current directory if the user didn't specify an output directory with --dir
+            let is_output_dir_provided = output_dir.is_some();
             let output_dir = if let Some(dir) = output_dir {
                 utils::create_dir_if_non_existent(&dir)?;
                 dir
@@ -174,22 +189,32 @@ pub fn run(
                 .zip(formats)
                 .zip(output_paths)
                 .try_for_each(|((input_path, formats), file_name)| {
-                    let output_file_path = output_dir.join(file_name); // Path used by single file format archives
-                    decompress_file(
-                        input_path,
+                    // Path used by single file format archives
+                    let output_file_path = if is_path_stdin(file_name) {
+                        output_dir.join("stdin-output")
+                    } else {
+                        output_dir.join(file_name)
+                    };
+                    decompress_file(DecompressOptions {
+                        input_file_path: input_path,
                         formats,
-                        &output_dir,
+                        output_dir: &output_dir,
                         output_file_path,
+                        is_output_dir_provided,
                         question_policy,
-                        args.quiet,
-                    )
-                })?;
+                        quiet: args.quiet,
+                        password: args.password.as_deref().map(|str| {
+                            <[u8] as ByteSlice>::from_os_str(str).expect("convert password to bytes failed")
+                        }),
+                        remove,
+                    })
+                })
         }
         Subcommand::List { archives: files, tree } => {
             let mut formats = vec![];
 
             if let Some(format) = args.format {
-                let format = parse_format(&format)?;
+                let format = parse_format_flag(&format)?;
                 for _ in 0..files.len() {
                     formats.push(format.clone());
                 }
@@ -215,9 +240,18 @@ pub fn run(
                     println!();
                 }
                 let formats = extension::flatten_compression_formats(&formats);
-                list_archive_contents(archive_path, formats, list_options, question_policy)?;
+                list_archive_contents(
+                    archive_path,
+                    formats,
+                    list_options,
+                    question_policy,
+                    args.password
+                        .as_deref()
+                        .map(|str| <[u8] as ByteSlice>::from_os_str(str).expect("convert password to bytes failed")),
+                )?;
             }
+
+            Ok(())
         }
     }
-    Ok(())
 }
